'--------------------------------------------
'Description : Dispensing fluids with a Linear Motor
'--------------------------------------------
'Ideally this is example is with a syringe (or something comparable
'that needs a cylinder moved translational) in combination with a 
'Pushbutton that is used for the duration of the dispensing
'
'Ideally you would use a state machine
'
'Procedure:
'    1.    Initialize the device
'    2.    Set the correct starting position
'    3.    Wait for a button to get pressed
'    4.    Then we start the dispensing and hold it for a minimum of 600ms
'    4.1   If we hit a set software limit or the button is released
'    5.    Stop dispensing
'    6.    Go back to the starting position
'    x.    Then jump back to 3.
'--------------------------------------------

DEF_TIM_VAR t                                                            'the variable of the timer is t

a = GETOBJ $6041.00                                                      'get CiA402 statusword
IF (a & $004) <> $004 THEN                                               'if CiA402 Operation enabled not set
   SETOBJ $6040.00 = 06                                                  'CiA402 - shutdown
   SETOBJ $6040.00 = 07                                                  'CiA402 - switch on
ENDIF

:InitializeHoming
   SETOBJ $6060.00=06                                                    'set the opmode to homing
   SETOBJ $6098.00=-1                                                    'set homing method to "homing on negative block with index pulse"
   SETOBJ $6099.01=10                                                    'seek velocity min^-1
   SETOBJ $6099.02=10                                                    'homing speed min^-1
   SETOBJ $609A.00=1                                                     'homing acceleration
   SETOBJ $607C.00=0                                                     'home offset is 0

   SETOBJ $6040.00=$001F                                                 'start homing via controlword

:CheckIfHomingIsComplete
   a = GETOBJ $6041.00                                                   'get statusword
   IF ((a & $1000) = 0) THEN GOTO CheckIfHomingIsComplete
                                                                         'if homing is not finished (set-point acknowledge is 0)
                                                                         'then ask for the statusword again
                                                                         'simply wait for it to finish
   SETOBJ $6060.00=03                                                    'set opmode to profile velocity mode
   SETOBJ $60FF.00=0                                                     'demand velocitiy = 0 mm/s
   SETOBJ $6040.00=$000F                                                 'CiA 402 Enable operation

'
:StartRequest
   a=GETOBJ $2311.02                                                     'get the physical digital input state
   IF ((a&$1)= 0) THEN GOTO StartRequest
                                                                         'if the button (DIGin 1) isn't activated
                                                                         'then we will wait until it's pressed
:start_dispensing
   SETOBJ $60FF.00=10                                                    'demand velocity = 10 mm/s
   START_TIM 600                                                         'set min. drive time = 600ms /// see 4.1 of this file

:LimitRequest
   a=GETOBJ $6064.00                                                     'get the actual position
   IF (a >= 100000) THEN GOTO GoHome
                                                                         'if the actual position is greater or equal than the limit position
   IF t<>1 THEN GOTO LimitRequest
                                                                         'if the min. drivetime is not finished we wont allow to unload and wait again
   a=GETOBJ $2311.02                                                     'get the physical digital input state
   IF ((a& $1) = $1) THEN GOTO LimitRequest
                                                                         'if the button is still pressed, we wont allow the unload

                                                                         'finally; all of the conditions above succeeded
:Unload
   SETOBJ $60FF.00=0                                                     'demand velocitiy = 0 mm/s
   SETOBJ $6060.00=01                                                    'set the opmode to profile position
   SETOBJ $607A.00=-1000                                                 'set the demand position
   SETOBJ $6040.00=$007F                                                 'start relative position with the 7th bit
   SETOBJ $6040.00=$000F                                                 'enable operation
   GOTO checkTargetReached

:GoHome
   SETOBJ $60FF.00=0                                                     'demand velocitiy = 0 mm/s
   SETOBJ $6040.00=$000F                                                 'CiA 402 Enable operation
   SETOBJ $6060.00=01                                                    'set the opmode to profile position
   SETOBJ $607A.00=0                                                     'set the demand position to the homing position
   SETOBJ $6040.00=$001F                                                 'start absolute position
   SETOBJ $6040.00=$000F                                                 'CiA 402 Enable operation


:CheckIfTargetIsReached
   a = GETOBJ $6041.00                                                   'get statusword
   IF ((a & $400)=0) THEN GOTO CheckIfTargetIsReached
                                                                         'if the target position is not reached
                                                                         'then wait and check again
   SETOBJ $6060.00=03                                                    'set the opmode to profile velocity
   SETOBJ $60FF.00=0                                                     'demand velocitiy = 0 mm/s to stop the motor
   SETOBJ $6040.00=$000F                                                 'CiA 402 Enable operation
   GOTO StartRequest                                                     'jump back and wait for the next request



