'--------------------------------------------      
'Description :   Lifting a load in combination with a brake and a limit switch
'--------------------------------------------
'This example is used with a load that is lifted with a motor and a reel.
'The motor shaft is connected to a brake that inhibits the shaft from
'rotating if no power (voltage) is supplied.
'The limit switch is activated when the load hits its lowest point
'
' Caution! DO NOT USE IN PRODUCTION
'
'Procedure:
'1.  Init the IO's: First the limit switch then a brake
'2.  Follow the state machine by increasing the step variable z
'2.z A subroutine is called for each step
' 	- check if everything is configured and enabled
' 	- find the lowest position and do homing
'	- move to start position
' 	- then do some position patterns

'--------------------------------------------
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
DEF_TIM_VAR t                   'timer variable t
LET t = 1                       't = 1 
LET w=1000                      'wait time in ms


LET z = 1                 		'variable: Used for the current state of the state machine
LET y = 0                 		'variable: if 1 then we need to wait for the timer, if 0 then no timer is active
LET x = 0               		'variable: if 1 then a current positioning is ongoing, if 0 then there is no action ongoing
'*************************************************************************

GOSUB initIO

START_TIM 500

:MainLoop
IF 1 = z THEN GOSUB initHome
IF 2 = z THEN GOSUB initHub
IF 3 = z THEN GOSUB enable_motor
IF 4 = z THEN GOSUB position_initial
IF 5 = z THEN GOSUB initPosSettings
IF 6 = z THEN GOSUB disable_motor

'here starts our step pattern
IF 7 = z THEN GOSUB enable_motor
IF 8 = z THEN GOSUB position_mid
IF 9 = z THEN GOSUB position_high2
IF 10 = z THEN GOSUB position_low2
IF 11 = z THEN GOSUB position_high1
IF 12 = z THEN GOSUB disable_motor
IF 13 = z THEN GOSUB enable_motor
IF 14 = z THEN GOSUB position_mid
IF 15 = z THEN GOSUB position_initial
IF 16 = z THEN GOSUB disable_motor
IF 17 = z THEN GOSUB endLoop

GOTO MainLoop

'initialize the digital I/O
:initIO
	SETOBJ $2310.$11=0       		'Set PLC I/O Level (24V)
	'Init Limit Switch
	SETOBJ $2310.$01=0				'reset the limit switches
	LET a = GETOBJ $2310.$01		'check the current limit switch input
	SETOBJ $2310.01= a | 1    		'set lower limit switch source to digital inputs
	SETOBJ $2310.03=$02       		'do a quickstop when the limit switch is connected
	'INIT Brake
	SETOBJ $2312.02=1  'aktivate the Brake on Digout 1
	SETOBJ $2312.03=1  'Hold Time 1 ms
RETURN

'initialize the homing position, set the parameters to harmlesser values than in the default application
:initHome
	a = GETOBJ $6041.00           	'check the CiA402 statusword
	IF (a & $004) <> $004 THEN    	'CiA402 operation enabled is not set
		SETOBJ $6040.00 = 06		'shutdown
		SETOBJ $6040.00 = 07      	'switch On
	ENDIF
	SETOBJ $607D.01=-20000      	'set the software min position limit to twice the height
	'configure homing
	SETOBJ $6060.00=06      		'set the mode of operation to homing
	SETOBJ $6098.00=17      		'set the homing method to home on negative limit switch
	SETOBJ $6099.01=10      		'set the seek velocity min^-1 to a lower value
	SETOBJ $6099.02=10      		'set the homing Speed min^-1 to a lower value
	SETOBJ $609A.00=50      		'set the homing acceleration to a lower value
	SETOBJ $607C.00=500    			'set the homing offset to 500
	'start homing       			'set the bit 4 of the controlword to
	SETOBJ $6040.00=$001F     		'start the homing via controlword
	z=z+1							'increase the step
RETURN

'
:initHub
IF y=0 THEN							'if no positioning ongoing	
   a=GETOBJ $6041.00
   IF ((a & $1400) = $1400) THEN  	'if the homing position is not reached and the homing is not done -> bit 12 & bit 10 of the CiA402 statusword
      SETOBJ $6060.$00=$01   		'set the mode of operation to profile position mode
      START_TIM 10					'wait a short time
      y = 1							'timer is active
   ENDIF
ELSE
   GOSUB CheckTimer					'check if the time passed
ENDIF
RETURN

'initialize our default application parameters
:initPosSettings
IF y=0 THEN							'if no positioning ongoing
   SETOBJ $607D.$01=-100  			'set the minimum software position limits
   SETOBJ $6083.$00=250   			'profile acceleration
   SETOBJ $6084.$00=250   			'profile deceleration
   START_TIM 100					'wait a short time
   y = 1							'timer is active
ELSE
   GOSUB CheckTimer					'check if the time passed
ENDIF
RETURN

'disable the motor
:disable_motor
IF y = 0 THEN						'if no positioning ongoing
      SETOBJ $6040.$00 = $07      	'CiA402 disable
      START_TIM 5000				'wait for 5 seconds
      y = 1							'timer is active
ELSE
      GOSUB CheckTimer				'check if the time passed
ENDIF
RETURN

'enable the motor and hold current position
:enable_motor
   IF y=0 THEN						'if no positioning ongoing
      a=GETOBJ $2360.$06   			'get actual position
      SETOBJ $607A.$00=a   			'set new position
      SETOBJ $6040.00=$001F  		'start absolutPosition
      SETOBJ $6040.00=$000F     	'CiA402 enable operation
      START_TIM 1000				'start the timer and wait
      y=1							'timer is active
   ELSE
      GOSUB CheckTimer				'check if the time passed
   ENDIF
RETURN

'lift just a little; our initial pattern position
:position_initial
IF x=0 THEN							'if no positioning ongoing
      SETOBJ $607A.$00=50     		'set new position
      SETOBJ $6040.00=$001F     	'start absolute position with CiA402 controlword
      SETOBJ $6040.00=$000F			'CiA402 enable operation
      w=3000						'wait time
      x = 1
    ELSE
       GOSUB checkTargetReached		'check if we reached
    ENDIF
RETURN

'lift half the way
:position_mid
   IF x=0 THEN						'if no positioning ongoing
      SETOBJ $607A.$00=4000     	'set new position
      SETOBJ $6040.00=$001F     	'start absolute position with CiA402 controlword
      SETOBJ $6040.00=$000F    	 	'CiA402 enable operation
      w=1500                    	'wait time
      x = 1
    ELSE
       GOSUB checkTargetReached		'check if we reached
    ENDIF
RETURN

'lift to the top
:PosHigh1
   IF x=0 THEN						'if no positioning ongoing
      SETOBJ $607A.$00=10500     	'set new position
      SETOBJ $6040.00=$001F     	'start absolute position with CiA402 controlword
      SETOBJ $6040.00=$000F     	'CiA402 enable operation
      w=3000                    	'wait time
      x = 1
    ELSE
       GOSUB checkTargetReached		'check if we reached
    ENDIF
RETURN

'lift almost to the top
:position_high2
   IF x=0 THEN						'if no positioning ongoing
      SETOBJ $607A.$00=9000     	'set new position
      SETOBJ $6040.00=$001F     	'start absolute position with CiA402 controlword
      SETOBJ $6040.00=$000F     	'CiA402 enable operation
      w=1000                    	'wait time
      x = 1
    ELSE
       GOSUB checkTargetReached		'check if we reached
    ENDIF
RETURN

'lift to the lower position
:position_low1
   IF x=0 THEN						'if no positioning ongoing
      SETOBJ $607A.$00=1000     	'set new position
      SETOBJ $6040.00=$001F     	'start absolute position with CiA402 controlword
      SETOBJ $6040.00=$000F     	'CiA402 enable operation
      w=500							'wait time
      x = 1
    ELSE
       GOSUB checkTargetReached		'check if we reached
    ENDIF
RETURN

'we are done with our step pattern, so we start with the first pattern again
:endLoop
 z = 7
RETURN


' Check if the target was reached
:checkTargetReached
IF y = 0 THEN						'is the timer enabled?
      a = GETOBJ $6041.00         	'get CiA402 statusword
      IF (a & $400) = $400 THEN   	'is the bit "Target Reached" set?
         START_TIM w              	'if yes, then we start the waiting timer
         y = 1						'timer is active
    ENDIF
ELSE								'timer is already enabled
   GOSUB CheckTimer					'check the timer
ENDIF
RETURN


'Check if the timer has been reached, if yes, increase z by one
:CheckTimer
IF t = 1 THEN
   y = 0
   x = 0
   z = z + 1
ENDIF
RETURN